Attribute VB_Name = "ListSM"
'  FILE:       ListSM.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functions to manipulate the list view
'              in the right-hand pane of the explorer-style frmMain.
'              Specifically, it contains functions to add nodes
'              to the list view and procedures to sync the list
'              control with the tree view control in the left-
'              hand pane.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "ListSM."

Private mListColumns As New cListColumns
Private Const miMinColIndex As Integer = 1
Private Function GetListNodeType(strKey As String) As Integer
    
    Dim iNodeType As Integer
    Dim lngWsp As Long

    iNodeType = TypeOfNode(strKey)
    
    If iNodeType = gintManagerStep Or iNodeType = gintWorkerStep Or iNodeType = gintStepLabel Then
        ' Check if the node being expanded is a step in a workspace that is being executed
        lngWsp = WorkspaceSelected(strKey)
        
        ' If so, we need to modify the node type
        If WorkspaceRunning(lngWsp) Then
            iNodeType = IIf(iNodeType = gintWorkerStep, gintRunWorker, gintRunManager)
        End If
    End If
    GetListNodeType = iNodeType

End Function
Private Function GetListColType(iNode As Integer) As ColType

    If (iNode = gintManagerStep Or iNode = gintWorkerStep Or _
            iNode = gintGlobalStep Or iNode = gintGlobalStepLabel Or iNode = gintStepLabel) Then
        GetListColType = StepProperties
    ElseIf (iNode = gintRunManager Or iNode = gintRunWorker) Then
        GetListColType = StepRun
    ElseIf (iNode = gintParameter Or iNode = gintParameterLabel Or _
            iNode = gintNodeParamExtension Or iNode = gintParamExtensionLabel Or _
            iNode = gintNodeParamBuiltIn Or iNode = gintParamBuiltInLabel) Then
        GetListColType = Parameter
    ElseIf (iNode = gintConnDtlLabel Or iNode = gintConnectionDtl) Then
        GetListColType = Connection
    ElseIf (iNode = gintNodeParamConnection Or iNode = gintParamConnectionLabel) Then
        GetListColType = ConnectionString
    Else
        GetListColType = Others
    End If

End Function

Private Sub LoadListNode(treAddTree As comctllib.TreeView, _
        lvwAddList As comctllib.ListView, _
        ByVal strKey As String, _
        ByVal intNodeType As Integer, _
        Optional ByVal intPosition As Integer)

    ' This procedure queryies the collection based on the node
    ' type and call a procedure to add a node to the list view
    
    Dim objNode As Object
    Dim lngIdentifier As Long
    
    On Error GoTo LoadListNodeErr
    
    If Not IsLabel(strKey) Then
        lngIdentifier = MakeIdentifierValid(strKey)
    End If
    
    ' Query the collection based on the node type to
    ' retrieve the properties for all the objects
    Select Case intNodeType
    
        Case gintWorkspace
        
        Case gintParameter, gintNodeParamExtension, gintNodeParamBuiltIn
            Set objNode = gcParameters.QueryParameter(lngIdentifier)
        
        Case gintNodeParamConnection
            Set objNode = gcConnections.QueryConnection(lngIdentifier)
        
        Case gintConnectionDtl
            Set objNode = gcConnDtls.QueryConnDtl(lngIdentifier)
        
        Case gintManagerStep, gintWorkerStep, gintGlobalStep
            Set objNode = gcSteps.QueryStep(lngIdentifier)
        
        Case Else
            ' A label node
    
    End Select
    
    ' Call a procedure to add the node to the list view
    Call AddListNode(treAddTree, lvwAddList, strKey, intNodeType, objNode)

    Set objNode = Nothing
    Exit Sub
    
LoadListNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "LoadListNode"
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadListNodeFailed, _
            gstrSource, _
            LoadResString(errLoadListNodeFailed)
    
End Sub
Public Sub AddListNode(treAddTree As comctllib.TreeView, _
        lvwAddList As comctllib.ListView, _
        ByVal strKey As String, _
        ByVal intNodeType As Integer, _
        ByVal objNode As Object, _
        Optional ByVal intPosition As Integer, _
        Optional ListNodeKey As String)

    ' This procedure adds a node to the list view for the
    ' passed in node key - the node properties are also
    ' passed in
    
    Dim mItem As ListItem
    Dim nodCurNode As Node
    
    On Error GoTo AddListNodeErr
    
    Set nodCurNode = treAddTree.Nodes(strKey)
    
    ' Add a node to the list view and obtain the key and text
    ' properties from the corresponding tree view node
    If intPosition >= 1 And intPosition <= lvwAddList.ListItems.Count + 1 Then
        Set mItem = lvwAddList.ListItems.Add(intPosition)
    Else
        Set mItem = lvwAddList.ListItems.Add
    End If
    If StringEmpty(ListNodeKey) Then
        mItem.Key = strKey
    Else
        mItem.Key = ListNodeKey
    End If
    
    If intNodeType = gintWorkspace Then
        mItem.Text = FilterWorkspaceName(nodCurNode.Text)
    Else
        mItem.Text = nodCurNode.Text
    End If
    
    mItem.SmallIcon = GetSmallIcon(strKey, intNodeType, objNode)
    mItem.Icon = mItem.SmallIcon
    
    ' Call a procedure to initialize the sub-items in the list
    ' view using the object properties
    Call InitSubItems(mItem, objNode, intNodeType)
    
    Set objNode = Nothing
    Exit Sub
    
AddListNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "AddListNode"
    On Error GoTo 0
    Err.Raise vbObjectError + errAddListNodeFailed, gstrSource, _
            LoadResString(errAddListNodeFailed)
    
End Sub
Private Sub InitSubItems(mListNode As comctllib.ListItem, _
        ByVal objNodeData As Object, _
        ByVal intNodeType As Integer)

    ' This procedure loads the sub-items in the passed in node
    ' based on the node type
    
    On Error GoTo InitSubItemsErr
    
    If Not IsLabel(mListNode.Key) And Not objNodeData Is Nothing Then
        Select Case intNodeType
        
            Case gintWorkspace
                ' No sub-items
            
            Case gintParameter, gintNodeParamExtension, gintNodeParamBuiltIn
                mListNode.SubItems(1) = objNodeData.ParameterValue
                mListNode.SubItems(2) = objNodeData.Description
            
            Case gintNodeParamConnection
                mListNode.SubItems(1) = objNodeData.ConnectionValue
                mListNode.SubItems(2) = objNodeData.Description
            
            Case gintConnectionDtl
                mListNode.SubItems(1) = objNodeData.ConnectionString
                mListNode.SubItems(2) = GetConnectionTypeString(objNodeData.ConnType)
            
            Case gintManagerStep, gintWorkerStep, gintGlobalStep
                mListNode.Tag = objNodeData.EnabledFlag
                Call SetChecked(frmMain.lvwStepView, mListNode, _
                        objNodeData.EnabledFlag)
                mListNode.SubItems(1) = objNodeData.StepLabel
                mListNode.SubItems(2) = objNodeData.StepTextFile
                mListNode.SubItems(3) = objNodeData.StepText
                mListNode.SubItems(4) = objNodeData.DegreeParallelism
            
            Case gintRunManager, gintRunWorker
                mListNode.SubItems(1) = objNodeData.Step.StepLabel
                If StringEmpty(objNodeData.Step.StepTextFile) Then
                    mListNode.SubItems(2) = objNodeData.Step.StepText
                Else
                    mListNode.SubItems(2) = objNodeData.Step.StepTextFile
                End If
                mListNode.SubItems(3) = GetInstanceStatus(objNodeData)
                If objNodeData.StartTime <> gdtmEmpty Then
                    mListNode.SubItems(4) = JulianDateToString(objNodeData.StartTime)
                Else
                    mListNode.SubItems(4) = gstrEmptyString
                End If
                If objNodeData.EndTime <> gdtmEmpty Then
                    mListNode.SubItems(5) = JulianDateToString(objNodeData.EndTime)
                Else
                    mListNode.SubItems(5) = gstrEmptyString
                End If
                mListNode.SubItems(6) = GetInstanceElapsedTime(objNodeData)
                mListNode.SubItems(7) = objNodeData.Step.IteratorName
                mListNode.SubItems(8) = GetInstanceItValue(objNodeData)
            
            Case Else
                BugAssert False, "Node type <> workspace, parameter, step or label!"
        
        End Select
    End If
    
    Exit Sub
    
InitSubItemsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "InitSubItems"
    On Error GoTo 0
    Err.Raise vbObjectError + errInitSubItemsFailed, _
            gstrSource, _
            LoadResString(errInitSubItemsFailed)
    
End Sub
Public Sub ProcessListViewDblClick(lvwNodes As ListView)
    ' Called when a DblClick event occurs on the list view control
    ' Since the list view does not have an Item_DblClick event,
    ' the listitem that the user is on is determined in the
    ' DblClick event of the list view and the properties for
    ' that node are displayed to the user
    
    Dim lngReturn As Long
    Dim intIndex As Long
    Dim mItem As comctllib.ListItem

    On Error GoTo ProcessListViewDblClickErr
    
    intIndex = lvwNodes.SelectedItem.Index
    
    ' If a list node has been selected by the user, call a
    ' procedure to display the node properties
    If intIndex >= 1 And intIndex <= lvwNodes.ListItems.Count Then
        Call DisplayProperties(lvwNodes.ListItems(intIndex).Key)
    End If
    
    Exit Sub
    
ProcessListViewDblClickErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ProcessListViewDblClick"
    Call ShowError(errUnableToProcessListViewDblClick)
    
End Sub

Public Sub UpdateListNode(treAddTree As comctllib.TreeView, _
        lvwAddList As comctllib.ListView, _
        ByVal strKey As String, _
        ByVal objNode As Object, _
        ByVal intNodeType As Integer)

    ' This procedure modifies the list view for the
    ' passed in node key - it loads the sub-items in the node
    ' from the passed in modified node data
    ' The tree view node must be initialized with the modified
    ' node data before this procedure is called
    
    Dim mItem As ListItem
    Dim nodCurNode As Node
    
    On Error GoTo UpdateListNodeErr
    
    Set mItem = lvwAddList.ListItems(strKey)
    
    ' Obtain the key and text properties from the corresponding
    ' tree view node
    Set nodCurNode = treAddTree.Nodes(strKey)
    
    If IsStep(intNodeType) Then
        mItem.SmallIcon = GetSmallIcon(strKey, intNodeType, objNode)
        mItem.Icon = mItem.SmallIcon
    End If
    
    If intNodeType = gintWorkspace Then
        mItem.Text = FilterWorkspaceName(nodCurNode.Text)
    Else
        mItem.Text = nodCurNode.Text
    End If
    
    ' Call a common procedure to initialize the sub-items
    ' collection of the node
    Call InitSubItems(mItem, objNode, intNodeType)
    
    Exit Sub
    
UpdateListNodeErr:
    ' Log the error code raised by Visual Basic
    If Err.Number = ccElemNotFound Then
        ' The node is not being displayed in the list view
        ' Just exit
        Exit Sub
    Else
        Call LogErrors(Errors)
        ' Not a critical error - display an error message and
        ' exit
        gstrSource = mstrModuleName & "UpdateListNode"
        ShowError errUpdateListNodeFailed
    End If
    
End Sub

Public Sub DeleteListNode(lvwAddList As comctllib.ListView, _
        ByVal strKey As String)

    ' This procedure deletes the passed in node from the
    ' list view if it exists
    
    On Error GoTo DeleteListNodeErr
    
    lvwAddList.ListItems.Remove strKey
    
    Exit Sub
    
DeleteListNodeErr:
    ' Log the error code raised by Visual Basic
    If Err.Number = ccElemNotFound Then
        ' The node is not being displayed in the list view
        ' Just exit
        Exit Sub
    Else
        Call LogErrors(Errors)
        ' Not a critical error - display an error message and
        ' exit
        gstrSource = mstrModuleName & "DeleteListNode"
        ShowError errUpdateListNodeFailed
    End If
    
End Sub
Private Sub MakeListColumns(lvwExplore As comctllib.ListView, ByVal intNodeType As Integer)
    ' Adds columns to the list view based on the type of node
    Dim iType As ColType
    Dim lTemp As Long
    
    On Error GoTo MakeListColumnsErr
    
    ' Save the list view width settings
    If lvwExplore.ColumnHeaders.Count >= miMinColIndex Then
        If Not StringEmpty(lvwExplore.ColumnHeaders(miMinColIndex).Tag) Then
            iType = CInt(lvwExplore.ColumnHeaders(miMinColIndex).Tag)
            
            For lTemp = miMinColIndex To lvwExplore.ColumnHeaders.Count
                mListColumns(iType, lTemp).ColWidth = lvwExplore.ColumnHeaders(lTemp).Width
            Next lTemp
        End If
    End If
    
    ' Clear the ColumnHeaders collection.
    lvwExplore.ColumnHeaders.Clear
    
    iType = GetListColType(intNodeType)
    For lTemp = mListColumns.lIndex(iType) To mListColumns.HIndex(iType)
        lvwExplore.ColumnHeaders.Add , , mListColumns(iType, lTemp).ColName
    Next lTemp
    ' Every node type has atleast one column header
    lvwExplore.ColumnHeaders(miMinColIndex).Tag = iType
    
    ' Restore the list view width settings if they have been saved previously
    If mListColumns.SettingSaved(iType) Then
        
        For lTemp = miMinColIndex To lvwExplore.ColumnHeaders.Count
            lvwExplore.ColumnHeaders(lTemp).Width = mListColumns(iType, lTemp).ColWidth
        Next lTemp
    End If
    
    Exit Sub
    
MakeListColumnsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "MakeListColumns"
    On Error GoTo 0
    Err.Raise vbObjectError + errMakeListColumnsFailed, gstrSource, _
            LoadResString(errMakeListColumnsFailed)

End Sub
Private Sub PopulateList(treExplore As comctllib.TreeView, _
        lvwExplore As comctllib.ListView, _
        ByVal strFirstKey As String, _
        Optional ByVal intNodeType As Integer = 0)
        
    ' This procedure populates the passed in list view with
    ' the details for all the siblings corresponding to the
    ' passed in node key
    
    Dim strKey As String
    Dim blnCheckBox As Boolean
    
    On Error GoTo PopulateListErr
    
    If intNodeType = 0 Then
        intNodeType = TypeOfNode(strFirstKey)
    End If
    
    blnCheckBox = ((intNodeType = gintManagerStep) Or (intNodeType = gintWorkerStep))
    
    ' Call a procedure to set the style property for the list view
    ' based on the flag initialized above
    Call SetListViewStyle(frmMain.lvwStepView, blnCheckBox)

    ' Add the data for the first node into the list view
    strKey = strFirstKey
    If intNodeType <> gintRunWorker And intNodeType <> gintRunManager Then
        Call LoadListNode(treExplore, lvwExplore, strKey, intNodeType)
    Else
        ' Load the details of all instances for the step
        ' in the list view
        Call LoadInstancesForStep(treExplore, lvwExplore, MakeIdentifierValid(strKey))
    End If
    
    ' For each sibling in the tree view,
    Do While strKey <> treExplore.Nodes(strKey).LastSibling.Key
    
        strKey = treExplore.Nodes(strKey).Next.Key
        If intNodeType <> gintRunWorker And intNodeType <> gintRunManager Then
            intNodeType = TypeOfNode(strKey)
        
            ' Call a procedure to load the details of the node
            ' into the list view
            Call LoadListNode(treExplore, lvwExplore, strKey, intNodeType)
        Else
            ' Call a procedure to load the details of all
            ' instances for the step in the list view
            Call LoadInstancesForStep(treExplore, lvwExplore, MakeIdentifierValid(strKey))
        End If
    
    Loop
    
    Exit Sub
    
PopulateListErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errPopulateListFailed, _
            mstrModuleName & "PopulateList", _
            LoadResString(errPopulateListFailed)
    
End Sub
Public Sub ExploreNode(treExplore As comctllib.TreeView, _
        lvwExplore As comctllib.ListView, _
        ByVal nodExpand As comctllib.Node)

    ' This procedure displays the details of the expanded node
    ' in the list view control
    
    Dim strKey As String
    Dim nodStart As Node
    Dim intNodeType As Integer
    
    On Error GoTo ExploreNodeErr
    
    ' Set the mousepointer to indicate Busy
    ShowBusy
    
    ' First clear all the existing items from the list view
    lvwExplore.ListItems.Clear
    
    strKey = nodExpand.Key
    intNodeType = GetListNodeType(strKey)
    
    ' Call a procedure to make the list of column headers
    ' depending on the node type
    Call MakeListColumns(lvwExplore, intNodeType)
        
    ' Initialize the tag of the list view to the key of the
    ' parent node that we are exploring
    lvwExplore.Tag = IIf(nodExpand Is Nothing, gstrEmptyString, nodExpand.Key)
    
    ' The details displayed will correspond to the children
    ' of the node being expanded, if any exist
    If treExplore.Nodes(strKey).Children > 0 Then
        Set nodStart = treExplore.Nodes(strKey).Child
    
        ' Initialize the key to contain the type of node that will
        ' be displayed in the list
        strKey = nodStart.Key
    
        If intNodeType <> gintRunWorker And intNodeType <> gintRunManager Then
            intNodeType = TypeOfNode(strKey)
        End If
        
        ' Call a procedure to display the details in the list
        Call PopulateList(treExplore, lvwExplore, strKey, intNodeType)
    End If
    
    ' Reset the mousepointer
    ShowFree
    
    Exit Sub
    
ExploreNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ExploreNode"
    Call ShowError(errExploreNodeFailed)
    
    ' Reset the mousepointer
    ShowFree
    
End Sub
Public Sub AddNodeToList(treStepView As TreeView, _
        lvwStepView As comctllib.ListView, _
        nodAddNode As comctllib.Node, _
        strKey As String, strStringToAdd As String, _
        objNode As Object, intNodeType As Integer)
    ' Adds a node to the list view control
    
    On Error GoTo AddNodeToListErr
    gstrSource = mstrModuleName & "AddNodeToList"
    
    Dim intPosition As Integer
    
    ' Check if the parent of the node that was added to the
    ' tree is the one that has been explored in the list view
    If Not treStepView.Nodes(strKey).Parent Is Nothing Then
        
        If lvwStepView.Tag = treStepView.Nodes(strKey).Parent.Key Then
            
            ' Find the position at which the node has to be added to
            ' the list
            If intNodeType = gintManagerStep Or intNodeType = gintWorkerStep Then
                intPosition = objNode.SequenceNo
            Else
                intPosition = 0
            End If
            
            ' Call a procedure to add the node to the list view
            Call AddListNode(treStepView, lvwStepView, _
                    strKey, intNodeType, objNode, intPosition)
        End If
    End If
    
    Exit Sub
    
AddNodeToListErr:
    Call LogErrors(Errors)
    ' Not a critical error - display an error message and
    ' exit
    gstrSource = mstrModuleName & "AddNodeToList"
    ShowError errUpdateListNodeFailed
    
End Sub

Public Sub CollapseNode(treExplore As comctllib.TreeView, _
        ByVal nodCollapse As comctllib.Node)

    ' This procedure does the necessary processing when
    ' the node is collapsed
    
    On Error GoTo CollapseNodeErr
    gstrSource = mstrModuleName & "CollapseNode"
    
    ' Set the mousepointer to indicate Busy
    ShowBusy
    
    ' No processing needed
    
    ' Reset the mousepointer
    ShowFree
    
    Exit Sub
    
CollapseNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "CollapseNode"
    Call ShowError(errCollapseNodeFailed)
    
    ' Reset the mousepointer
    ShowFree
    
End Sub
Public Sub RefreshView(treExplore As comctllib.TreeView, _
        lvwExplore As comctllib.ListView)

    ' This procedure is called when the user selects the View
    ' Refresh option. It refreshes the data in the list view.
    
    Dim strKey As String
    Dim nodToExpand As Node
    Dim intNodeType As Integer
    Dim mItem As ListItem
    
    On Error GoTo RefreshViewErr
    gstrSource = mstrModuleName & "RefreshView"
    
    ' Set the mousepointer to indicate Busy
    ShowBusy
    
    If lvwExplore.ListItems.Count > 0 Then
    
        ' The parent of any node being displayed in the list
        ' view control is the one that needs to be refreshed
        Set mItem = lvwExplore.ListItems.Item(lvwExplore.ListItems.Count)
        strKey = mItem.Key
        
        Set nodToExpand = treExplore.Nodes(strKey).Parent
        
        ' Call a procedure to expand the parent node data
        Call ExploreNode(treExplore, lvwExplore, nodToExpand)
        
    End If
    
    ' Reset the mousepointer
    ShowFree
    
    Exit Sub
    
RefreshViewErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "RefreshView"
    Call ShowError(errRefreshViewFailed)
    
    ' Reset the mousepointer
    ShowFree
    
End Sub
Public Sub ProcessTreeClick(treExplore As comctllib.TreeView, _
        lvwExplore As comctllib.ListView)

    ' This procedure is called when the user selects the Explore
    ' option on the context-sensitive menu. It displays
    ' details of all the child nodes of the tree view node
    ' selected by the user in the list view.
    ' It does not expand the currently selected node in the
    ' tree view - consistent with the behaviour of the Windows
    ' explorer
    
    Dim strKey As String
    Dim nodToExpand As Node
    Dim intNodeType As Integer
    
    On Error GoTo ProcessTreeClickErr
    
    ShowBusy
    
    strKey = ItemSelectedInTree(treExplore)
    
    If Not StringEmpty(strKey) Then
    
        ' The node that has been selected by the user should
        ' be expanded
        Set nodToExpand = treExplore.Nodes(strKey)
        
        ' Call a procedure to expand the node data in the
        ' list view
        Call ExploreNode(treExplore, lvwExplore, nodToExpand)
        
        ' Call a procedure to add all application extensions defined for the workspace
        ' to the Tools menu
        Call ModifyToolsMenu(WorkspaceSelected(strKey))
    End If
    
    ShowFree
    
    Exit Sub
    
ProcessTreeClickErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ProcessTreeClick"
    Call ShowError(errExploreFailed)
    
End Sub
Public Function GetExpandedIcon(ByVal intNodeType As Integer, _
        Optional objNode As Object = Nothing, _
        Optional bEnabled As Boolean = False) As TreeImages
        
    ' This procedure returns the index of the image that is
    ' to be displayed when the passed in node type is expanded
    
    Dim intImageIndex As Integer
    
    On Error GoTo GetExpandedIconErr
         
    Select Case intNodeType
        Case gintWorkspace
            intImageIndex = gintImageWorkspaceOpen
        
        Case gintManagerStep
            If objNode Is Nothing Then
                intImageIndex = IIf(bEnabled, gintImageManagerOpenEn, gintImageManagerOpenDis)
            Else
                intImageIndex = IIf(objNode.EnabledFlag, gintImageManagerOpenEn, gintImageManagerOpenDis)
            End If
                
        Case gintWorkerStep
            If objNode Is Nothing Then
                intImageIndex = IIf(bEnabled, gintImageWorkerEn, gintImageWorkerDis)
            Else
                intImageIndex = IIf(objNode.EnabledFlag, gintImageWorkerEn, gintImageWorkerDis)
            End If
        
        Case gintGlobalStep
            intImageIndex = gintImageGlobalOpen
        
        Case gintParameter, gintNodeParamConnection, gintConnectionDtl, _
                gintNodeParamExtension, gintNodeParamBuiltIn
            ' Cannot have any sub-nodes. No change in icon.
            intImageIndex = gintImageParameter
                
        Case Else
            ' A label node
            intImageIndex = gintImageLabelOpen
            
    End Select
    
    GetExpandedIcon = intImageIndex
    Exit Function
    
GetExpandedIconErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Not a critical error - set a default return image index
    ' and exit
    GetExpandedIcon = gintImageLabelOpen
    
End Function
Public Function GetCollapsedIcon(ByVal intNodeType As Integer, _
        Optional objNode As Object = Nothing, _
        Optional bEnabled As Boolean = False) As TreeImages
    
    ' This procedure returns the index of the image that is
    ' to be displayed when the passed in node type is collapsed
    
    Dim intImageIndex As Integer
    
    On Error GoTo GetCollapsedIconErr
         
    Select Case intNodeType
        Case gintWorkspace
            intImageIndex = gintImageWorkspaceClosed
        
        Case gintManagerStep
            If objNode Is Nothing Then
                intImageIndex = IIf(bEnabled, gintImageManagerClosedEn, gintImageManagerClosedDis)
            Else
                intImageIndex = IIf(objNode.EnabledFlag, gintImageManagerClosedEn, gintImageManagerClosedDis)
            End If
                
        Case gintWorkerStep
            If objNode Is Nothing Then
                intImageIndex = IIf(bEnabled, gintImageWorkerEn, gintImageWorkerDis)
            Else
                intImageIndex = IIf(objNode.EnabledFlag, gintImageWorkerEn, gintImageWorkerDis)
            End If
        
        Case gintGlobalStep
            intImageIndex = gintImageGlobalClosed
        
        Case gintParameter, gintNodeParamConnection, gintNodeParamExtension, gintNodeParamBuiltIn
            ' Cannot have any sub-nodes. No change in icon.
            intImageIndex = gintImageParameter
                
        Case Else
            ' A label node
            intImageIndex = gintImageLabelClosed
            
    End Select
    
    GetCollapsedIcon = intImageIndex
    Exit Function
    
GetCollapsedIconErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Not a critical error - set a default return image index
    ' and exit
    GetCollapsedIcon = gintImageLabelClosed
    
End Function
Public Function GetSmallIcon(ByVal strNodeKey As String, _
        ByVal intNodeType As Integer, _
        ByVal objNode As Object) As TreeImages
    
    ' This procedure returns the index of the image that is
    ' to be displayed in the list view
    
    On Error GoTo GetSmallIconErr
         
    ' Initialize the SmallIcon depending on the node type
    If IsLabel(strNodeKey) Then
        GetSmallIcon = gintImageLabelClosed
    Else
        Select Case intNodeType
        
            Case gintWorkspace
                GetSmallIcon = gintImageWorkspaceClosed
            
            Case gintParameter, gintNodeParamConnection, gintNodeParamExtension, _
                    gintNodeParamBuiltIn, gintConnectionDtl
                GetSmallIcon = gintImageParameter
            
            Case gintManagerStep, gintRunManager
                GetSmallIcon = IIf(objNode.EnabledFlag, gintImageManagerClosedEn, gintImageManagerClosedDis)
            
            Case gintWorkerStep, gintRunWorker
                GetSmallIcon = IIf(objNode.EnabledFlag, gintImageWorkerEn, gintImageWorkerDis)
            
            Case gintGlobalStep
                GetSmallIcon = gintImageGlobalClosed
            
            Case Else
                BugAssert False, "Node type <> workspace, parameter, step or label!"
        
        End Select
    End If
    
    GetSmallIcon = GetSmallIcon
    Exit Function
    
GetSmallIconErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Not a critical error - set a default return image index
    ' and exit
    GetSmallIcon = gintImageLabelClosed
    
End Function
